# this script is designed to be used with the liveplot tcl script in the 
# gsasgui package. It displays entries from the ICDD/JCPDS database on
# top of the GSAS difference plot.
#
# Brian Toby (NIST) 5/98
#
# where are we?
set scriptname [info script]
# translate links -- go six levels deep
foreach i {1 2 3 4 5 6} {
    if {[file type $scriptname] == "link"} {
	set link [file readlink $scriptname]
	if { [file  pathtype  $link] == "absolute" } {
	    set scriptname $link
	} {
	    set scriptname [file dirname $scriptname]/$link
	}
    } else {
	break
    }
}
set icdddir [file dirname $scriptname]
set getcard [file join $icdddir pdf2peaks.tcl]
set flag 0
if ![file exists $getcard] {
    puts "file pdf2peaks.tcl not found"
    set flag 1
}
set logicshell [file join $icdddir tcl_logic]
if ![file exists $logicshell] {
    set logicshell [file join $icdddir tcl_logic.exe]
}
if ![file exists $logicshell] {
    set logicshell [exec which tclsh]
}
if ![file exists $logicshell] {
    puts "file no tclsh shell for pdf2peaks.tcl"
    set flag 1
}
if !$flag {
    catch {
	pack [menubutton .a.peaks -text "Peak Gen" \
		-underline 0 -menu .a.peaks.menu] \
		-side left    
	menu .a.peaks.menu
    }
    catch {
	.a.peaks.menu add command -label "Plot ICDD Card" \
		-command icddbox
    }
    set cellparm(Imult) 0.0
    set cellparm(ymin) 0
    set cellparm(ymax) 1000
}

proc icddbox {} {
    set bx .opticdd
    catch {destroy $bx}
    toplevel $bx
    wm iconname $bx "ICDD options"
    wm title $bx "ICDD options"

    pack [frame $bx.0  -bd 3 -relief groove] -side top
    pack [label $bx.0.0 -text "ICDD card number" ] -side left
    pack [entry $bx.0.1 -width 7 -textvariable icddnum ] -side left
    pack [frame $bx.1  -bd 3 -relief groove] -side top
    pack [label $bx.1.0 -text "Wavelength" ] -side left
    pack [entry $bx.1.1 -width 9 -textvariable cellparm(wavelength)] -side left
    pack [frame $bx.a -bd 4 -relief groove] -side top
    pack [frame $bx.a.a] -side top -anchor w
    pack [label $bx.a.a.1 -text "Int  = Min:"] -side left
    pack [entry $bx.a.a.2 -width 4 -textvariable cellparm(ymin)] -side left
    pack [frame $bx.a.b] -side top
    pack [label $bx.a.b.1 -text "(or) = I * "] -side left
    pack [entry $bx.a.b.2 -width 5 -textvariable cellparm(Imult)] -side left
    pack [label $bx.a.b.3 -text "+ Max: "] -side left
    pack [entry $bx.a.b.4 -width 6 -textvariable cellparm(ymax)] -side left
    pack [frame $bx.b] -side top
    pack [button $bx.b.1 -command posticdd -text "Plot lines"] \
	    -side left
    pack [button $bx.b.2 -command "deleteicdd $bx" -text "Delete lines"] \
	    -side left
}

proc deleteicdd {{bx ""}} {
    global box
    catch {$box element delete ICDD}
    bind $box <Shift-Button-1> ""
    if {$bx != ""} {destroy $bx}
}

proc posticdd {} {
    global icddnum getcard logicshell cellparm box

    deleteicdd
    if [catch {expr $icddnum}] {
	tk_messageBox -message "Error: $icddnum is an invalid entry number" \
		-type ok -icon warning
	return
    }
    if [catch {expr $cellparm(ymin)}] {
	tk_messageBox -message "Error: Int Min value ($cellparm(ymin)) is an invalid number" \
		-type ok -icon warning
	return
    }
    if [catch {expr $cellparm(ymax)}] {
	tk_messageBox -message "Error: Int Max value ($cellparm(ymax)) is an invalid number" \
		-type ok -icon warning
	return
    }
    if [catch {expr $cellparm(Imult)}] {
	tk_messageBox -message "Error: Int Mult value ($cellparm(Imult)) is an invalid number" \
		-type ok -icon warning
	return
    }
    if [catch {set list [exec $logicshell $getcard $icddnum]} errmsg] {
	tk_messageBox -message "ICDD/JCPDS Error: $errmsg" \
		-type ok -icon warning
	return
    }
    if [catch {eval $list} errmsg] {
	tk_messageBox -message \
		"pdf2peaks error: $errmsg (this should not happen)" \
		-type ok -icon warning
	return
    }
    set data dgen1
    global $data
    initicddpeaks  $data
    set ${data}(h) $h
    set ${data}(k) $k
    set ${data}(l) $l
    set ${data}(dspaces) $d
    set ${data}(wavelength) $cellparm(wavelength)
    set pi 3.14159
    set todeg [expr 360. / $pi]
    foreach dspace $d i $int {
	lappend ${data}(extinctions) 0
	set asinarg [expr $cellparm(wavelength) * 0.5 / $dspace]
	if {$asinarg <= 1} {
	    set tt [expr $todeg * asin ($asinarg)]
	    lappend ${data}(x) $tt
	    lappend xvec $tt $tt $tt
	    set ymax [expr ($i * $cellparm(Imult)) + $cellparm(ymax)]
	    lappend yvec $cellparm(ymin) $ymax $cellparm(ymin)
	} else {
	    lappend ${data}(x) -1.
	}
    }
    set xnam [set ${data}(ttvector)]
    $xnam set  [set ${data}(x)]
    $box element create ICDD \
	    -label $icddnum \
	    -color cyan -linewidth 2  -symbol {} \
	    -xdata $xvec -ydata $yvec
    bind $box <Shift-Button-1> "lblicddhkl %W %x"
}

proc lblicddhkl {plot x} {
    global cellparm command blt_version
    # look for peaks within pixelregion pixels
    set pixelregion 5
    set data dgen1
    if {$data == ""} return
    global $data
    set xmin [$plot xaxis invtransform [expr $x - $pixelregion]]
    set xmax [$plot xaxis invtransform [expr $x + $pixelregion]]
    set xnam [set ${data}(ttvector)]
    set peaknums [$xnam search $xmin $xmax]
    set peaklist {}
    set ymax [lindex [$plot yaxis limits] 1]
    set xcen 0    
    foreach peak $peaknums {
	set big 0
	foreach i {h k l} {
	    set $i [lindex [set ${data}($i)] $peak]
	    if {[set $i] < -9 || [set $i] > 9} {set big 1}
	}
	set xcen [expr $xcen + [$xnam range $peak $peak]]
	set sym {}
	if [lindex [set ${data}(extinctions)] $peak] {set sym e}
	if $big {
	    lappend peaklist $h,$k,$l$sym
	} else {
	    lappend peaklist $h$k$l$sym
	}
    }
    if {$peaklist == ""} return
    set xcen [expr $xcen / [llength $peaknums]]
    # avoid bug in BLT 2.3 where Inf does not work for text markers
    if {$blt_version == 2.3} {
	set ycen [lindex [$plot yaxis limits] 1]
    } else  {
	set ycen Inf
    }
    set mark [$plot marker create text -coords "$xcen $ycen" \
	    -rotate 90 -text $peaklist -anchor n -bg ""]
    after 10000 "$plot marker delete $mark"
}

proc initicddpeaks {data} {
    global $data ${data}_x ${data}_y ${data}_tt
    # define vectors for the data
    set ${data}(type) peaks
    set ${data}(xvector) ${data}_x
    catch {vector ${data}_x}
    set ${data}(yvector) ${data}_y
    catch {vector ${data}_y}
    set ${data}(ttvector) ${data}_tt
    catch {vector ${data}_tt}
    # initialize arrays
    set ${data}(x) ""
    set ${data}(h) {}
    set ${data}(k) {}
    set ${data}(l) {}
    set ${data}(dspaces) {}
    set ${data}(wavelength) ""
    set ${data}(xunits) ""
    set ${data}(xlabel) ""
    set ${data}(ylabel) ""
    set ${data}(xclabel) ""
    set ${data}(yclabel) ""
    set ${data}(spg) ""
    set ${data}(extcodes) ""
}
