# Where are the rest of the files -- assume that they are in the same 
# location as this script, but translate up to 20 levels of links 
set scriptname [info script]
set i -1
while {[file type $scriptname] == "link"} {
    if {[incr i] >= 20} {
	puts "More than $i links for [info script], giving up"
	destroy .
    }
    if {[file pathtype [set link [file readlink $scriptname]]] == "absolute"} {
	set scriptname $link
    } {
	set scriptname [file dirname $scriptname]/$link
    }
}
set settings(home) [file dirname $scriptname]

# load logic if not loaded
lappend auto_path $settings(home)
package require logic

# size of pdf1/pdf2 font
set settings(font) "Courier"
# size of pdf2 window
set settings(pdf2width) 120
# use cat in pipes on unix
#set settings(errorpipe) {}
catch { # tcl_platform 
#    if {$tcl_platform(platform) == "unix"} {set settings(errorpipe) "|& cat"}
    if {$tcl_platform(platform) == "windows"} {set settings(pdf2width) 93}
}
set  settings(showaids) 0
bind all <Control-KeyPress-c> {destroy .}
source $settings(home)/macros.tcl
source $settings(home)/filemenu.tcl
source $settings(home)/periodic.tcl
# initial settings
set settings(peaktype) {}
set settings(wave) {}
set settings(temp) d
set settings(peakorder) {}
set settings(options) { {seq #} {entry #} formula {chem name} \
	{cmn name} {min name} {reference}  peaks }
set settings(flag)    { seq      -entry  -formula -chemname \
	-common    -mineral   -reference  -peaks }
if {$tcl_platform(platform) != "windows"} {
    set settings(printer) {lpr}
} else {
    set settings(printer) notepad
}

foreach val {1 2 3 4 5 6} {
    set settings(ent$val) [lindex $settings(options) [expr $val - 1]]
    set settings(len$val) 0
}
set settings(len1) 6
set settings(len2) 6
foreach var {1 2 3 4 5 6 7 8} {
    set peakopt(peak$var) ""
    set peakopt(err$var) ""
}
set peakopt(err1) "0.05"
set peakopt(temp) d
if [file exists ~/.logicrc] {
    if [catch {source  ~/.logicrc} errmsg] {
	tk_dialog .warn "Warning" "Error reading ~/.logicrc: $errmsg" {} 0 OK
    }
}

# create the boxes
wm title . "Program Logic"
frame .l1 -relief groove -bd 4
label .l2 -text <>
frame .l3
pack .l1 -side top -fill x
pack .l2 -anchor e -side top 
pack .l3 -side top -fill both -expand yes
text  .l3.t -relief raised -bd 2 -height 10 -width 50 \
	-wrap none \
	-yscrollcommand ".l3.s set" \
	-xscrollcommand ".l3.a set" 
scrollbar .l3.s -command ".l3.t yview" 
scrollbar .l3.a -command ".l3.t xview" -orient horizontal
pack .l3.a -side bottom -fill x
pack .l3.t -side left -fill both -expand yes
pack .l3.s -side left -fill y 

foreach bar {file view search options} {
    menubutton .l1.$bar -text $bar -underline 0 \
	    -menu .l1.$bar.menu
    pack .l1.$bar -side left
    menu .l1.$bar.menu
}

update_logic

# file menu
set menu .l1.file.menu
$menu add command -label  Init -command {initsub_menu .initsub}
$menu add command -label  Save -command {savefile_menu .save} 
$menu add command -label  Reload -command {loadfile_menu .load} 
$menu add command -label  Quit -command {destroy .}

# view menu
set menu .l1.view.menu
$menu add command -label  Browse -command {make_browse 1}
$menu add command -label  Index -command {make_index} 

# search menu
set menu .l1.search.menu
$menu add command -label  Subfile -command {loadsub_menu .loadsub} 
$menu add command -label  Chemistry -command {element_menu .elemsub} 
$menu add command -label "Element count" -command {elemcount_menu .countsub}
$menu add command -label  Peaks -command {peak_menu .peak}
$menu add command -label  Undo -command {undo} 
set settings(undo) "$menu  entryconfigure Undo"
eval $settings(undo) -state disabled

# options menu
set menu .l1.options.menu
$menu add command -label "Display settings" -command {settings_menu .set} 
$menu add command -label "Index contents" -command {indexsettings_menu .set}
$menu add command -label  "Printer settings" -command {printsettings_menu .set}
if {$tcl_version >= 8.0} {
    $menu add cascade -label "Font" -menu $menu.font
    menu $menu.font
    foreach size {5 6 7 8 9 10 11 12 13 14 15 16} {
	$menu.font add command -label "Courier $size" \
		-command "set settings(font) \"Courier $size\"; \
		catch {\$txtbx config -font \$settings(font)}; \
		catch {\$indexbx config -font \$settings(font)}"
    }
}
$menu add command -label  SaveSettings -command savesettings

# display copyright notice
if [catch {set shownotice [file exists ~/icdd.not]}] {set shownotice 1}
if [catch {set havenotice [file exists $settings(home)/copynot.txt]}] {
    set havenotice 0
}
if { $havenotice && !$shownotice} {
    set in [ open [file join $settings(home) copynot.txt]]
    set msg [read $in]
    close $in
    if [tk_dialog .warn "Copyright Notice" \
	    $msg {} 0 OK Quit] {destroy .} {catch {close [open ~/icdd.not w]}}
}

