# center a new window (w) on window wp (defaults to .)
proc centerontop {w "wp ."} {
    wm withdraw $w
    update idletasks
    # center the new window in the middle of the parent
    set x [expr [winfo x $wp] + [winfo width $wp]/2 - \
	    [winfo reqwidth $w]/2 - [winfo vrootx $wp]]
    if {$x < 0} {set x 0}
    set xborder 10
    if {$x+[winfo reqwidth $w] +$xborder > [winfo screenwidth $w]} {
	incr x [expr \
		[winfo screenwidth $w] - ($x+[winfo reqwidth $w] + $xborder)]
    }
    set y [expr [winfo y $wp] + [winfo height $wp]/2 - \
	    [winfo reqheight $w]/2 - [winfo vrooty $wp]]
    if {$y < 0} {set y 0}
    set yborder 25
    if {$y+[winfo reqheight $w] +$yborder > [winfo screenheight $w]} {
	incr y [expr \
		[winfo screenheight $w] - ($y+[winfo reqheight $w] + $yborder)]
    }
    wm geom $w +$x+$y
    wm deiconify $w
}

proc warnbox {message} {
    # kill any window with this name
    catch {destroy .warn}
    toplevel .warn
    pack [message .warn.1 -width 8c -justify left -bd 2 \
	    -text $message]          
    pack [button .warn.2 -text OK -command {destroy .warn}] -side bottom
    centerontop .warn
}

proc waitbox {message} {
    # kill any window with this name
    catch {destroy .wait}
    toplevel .wait
    pack [message .wait.1 -width 8c -justify left -bd 6 -relief raised \
	    -text $message]          
    centerontop .wait
}

#----------------------------------------------------------------
#create a commands from the menu results
#----------------------------------------------------------------
proc makesubcommand {logic listval} {
    if {$logic == "all" || $logic == "none"} {
	saveundo
	if {$logic == "all"} {catch {destroy .index .browse}}
	loadsub -$logic
	update_logic
	return
    }
    if {$listval == ""} return
    saveundo
    if {$logic == "or" || $logic == "replace" \
	    || $logic == "all" } {catch {destroy .index .browse}}
    loadsub -$logic -subfile [expr 1+$listval]
    update_logic
}

proc makeloadmapcommand {} {
    global fileselect file_result
    saveundo
    loadmap $fileselect(path) -$file_result
    update_logic
}

proc makecountcommand {logic} {
    global countvar
    if {$logic == ""} return
    set countlist {}
    for {set i 0} {$i < 10} {incr i} {
	if {$countvar($i)} {lappend countlist $i}
    }
    if {$countlist == {}} return
    saveundo
    elemcount -count $countlist -$logic
    update_logic
}

proc makeelementcommand {logic req_opt opt_opt req_list opt_list} {
    if {$logic == ""} return
    if {$req_opt == ""} return
    if {$opt_opt == ""} return
    set list1 [list [parselem $req_list]]
    saveundo
    if {$opt_opt == "alloptional"} {
	eval element -$logic -$req_opt $list1 -$opt_opt
    } else {
	set list2 [list [parselem $opt_list]]
	eval element -$logic -$req_opt $list1 -$opt_opt $list2
    }
    update_logic
}

#----------------------------------------------------------------
# multi-purpose macros
#----------------------------------------------------------------
proc options_list {frame {optvar result} {listtype load}} {
    global $optvar
    if {$listtype == "load"} {
	pack [ label $frame.label -text "Combine tables by:" \
		] -side top -anchor w
	set options {"OR file with previous" "AND file with previous" \
		"REMOVE file from previous" \
		"REMOVE previous from file" "REPLACE previous with file"}
	set qual {or and remove not replace}
	set default replace
    } elseif {$listtype == "search"} {
	pack [ label $frame.label -text "Combine tables by:" \
		] -side top -anchor w
	set options {"OR search with previous" "AND search with previous" \
		"REMOVE search from previous" \
		"REMOVE previous from search"}
	set qual {or and remove not}
	set default and
    } else {
	set options {"set table to all entries" \
		"clear all entries from table"}
	set qual {all none}
	set default all
    }
    set optnum 0
    foreach item $options {
	set name [lindex $qual $optnum]
	pack [ radiobutton $frame.$name -variable $optvar \
		-text $item -value $name ] -side top -anchor w
	incr optnum
    }
    # set default option -- and except in init manu
    set $optvar $default
}

proc logic_list {frame} {
    # create a listbox
    set listbox \
	    [listbox $frame.list -yscrollcommand \
	    "$frame.scroll set" -selectmode single]
    # create the scroll bar
    set scroll \
	    [scrollbar $frame.scroll \
	    -command "$frame.list yview"]
    # add a line for each entry from the sublist command
    foreach item [sublist] { $listbox insert end $item }
    pack $frame.list $frame.scroll -side left -fill y
    # start with the first entry set
    $frame.list selection set 1
}

#----------------------------------------------------------------
#----------------------------------------------------------------
proc loadsub_menu {top} {
    global load_result leftbox
    # reuse any window with this name
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    
    wm title $top "Load predefined subfiles"
    set topbox [frame $top.a]; # frame for all boxes
    set botbox [frame $top.b]; # frame for buttons
    pack $topbox $botbox -side top
    #
    # create frame for the options menu and then the menu
    set rightbox [frame $top.a.option -relief raised -bd 2]
    options_list $rightbox load_result load
    # create frame for the subfile menu and then the menu
    set leftbox [frame $top.a.sublist -relief raised -bd 2] 
    logic_list $leftbox
    pack $rightbox $leftbox -side right
    
    button $botbox.doit -text Apply \
	    -command {makesubcommand $load_result [$leftbox.list curselection]}
    button $botbox.end -text Close -command "destroy $top"
    pack $botbox.doit $botbox.end -side left
}

proc initsub_menu {top} {
    global init_result
    # reuse any window with this name
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    wm title $top "Initialize table"
    set topbox [frame $top.a]; # frame for all boxes
    set botbox [frame $top.b]; # frame for buttons
    pack $topbox $botbox -side top
    #
    # create frame for the options menu and then the menu
    set rightbox [frame $top.a.option -relief raised -bd 2]
    options_list $rightbox init_result init
    pack $rightbox -side left
    
    button $botbox.doit -text Apply -command \
	"makesubcommand \$init_result 0; destroy $top"
    button $botbox.end -text Close -command "destroy $top"
    pack $botbox.doit $botbox.end -side left
    centerontop $top
}

proc loadfile_menu {top} {
    global fileselect file_result
    # destroy any window with this name
    catch {destroy $top}
    set fileselect(top) [toplevel $top -class Fileselect]
    wm title $top "Load table from file"
    set fileselect(command) {makeloadmapcommand}
    fileselect {Select a file to load} {*.pdf} 
    options_list [frame $top.logic -bd 2 -relief groove] file_result load
    $fileselect(top).fileselect.buttons.ok.b config -text Read
    pack $top.logic -side right
    centerontop $top
}

proc savefile_menu {top} {
    global fileselect
    # destroy any window with this name
    catch {destroy $top}
    set fileselect(top) [toplevel $top -class Fileselect]
    wm title $top "Save table on file"
    set fileselect(command) {savemap $fileselect(path)}
    fileselect {Enter/Select a file to write} {*.pdf} 0
    $fileselect(top).fileselect.buttons.ok.b config -text Write
    centerontop $top
}

proc elemcount_menu {top} {
    global countvar count_result
    # reuse any window with this name
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    wm title $top "Select by number of elements"
    set topbox [frame $top.a]; # frame for all boxes
    set botbox [frame $top.b]; # frame for buttons
    pack $topbox $botbox -side top
    # create frame for the options menu and then the menu
    set rightbox [frame $top.a.option -relief raised -bd 2]
    options_list $rightbox count_result search
    # create frame for the subfile menu and then the menu
    set leftbox [frame $top.a.sublist -relief raised -bd 2] 
    pack $rightbox $leftbox -side right

    for {set i 0} {$i < 10} {incr i} {
	pack [checkbutton $leftbox.$i -variable countvar($i) \
		-text "$i elements"] -side top -anchor w
    }

    button $botbox.doit -text Apply -command {makecountcommand $count_result}
    button $botbox.end -text Close -command "destroy $top"
    pack $botbox.doit $botbox.end -side left
}

proc element_menu {top} {
    global elem_result rightbox
    # reuse any window with this name
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    wm title $top "Select by composition"
    set topbox [frame $top.a]; # frame for all boxes
    set botbox [frame $top.b]; # frame for buttons
    pack $topbox $botbox -side top
    # create frame for the options menu and then the menu
    set rightbox [frame $top.a.option -relief raised -bd 2]
    options_list $rightbox elem_result search
    set leftbox [frame $top.a.l] 
    pack $rightbox $leftbox -side left
    set lefttop [frame $top.a.ltop -relief raised -bd 2] 
    set leftbot [frame $top.a.lbot -relief raised -bd 2] 
    pack $lefttop $leftbot -in $leftbox -side top
    pack [label $lefttop.0 -text "Required elements"] -side top
    pack [radiobutton $lefttop.1 -variable required_opt -value oneof\
		-text "One of:"] -side top -anchor w
    pack [radiobutton $lefttop.2 -variable required_opt -value allof\
		-text "All of:"] -side top -anchor w
    pack [entry $lefttop.3 -textvariable  req_list]  -side top 
    pack [button $lefttop.4 -text "Use table" \
	    -command "makePeriodicTable .req req_list {Required Elements}"]\
	    -side top -anchor center
    $lefttop.1 invoke

    pack [label $leftbot.0 -text "Optional elements"] -side top
    pack [radiobutton $leftbot.1 -variable optional_opt -value alloptional\
	    -command "hideelembox 1 $leftbot" -text "All elements optional"]\
	    -side top -anchor w
    pack [radiobutton $leftbot.2 -variable optional_opt  -value optional\
	    -command "hideelembox 0 $leftbot" -text "Allowed elements:"]\
	    -side top -anchor w
    pack [entry $leftbot.3 -textvariable  opt_list]  -side top 
    pack [button $leftbot.4 -text "Use table" \
	    -command "makePeriodicTable .opt opt_list {Optional Elements}"] \
	    -side top -anchor center
    $leftbot.1 invoke
    button $botbox.doit -text Apply -command {
	makeelementcommand \
		$elem_result $required_opt $optional_opt $req_list $opt_list}
    button $botbox.end -text Close \
	    -command "destroy $top; catch {destroy .req}; catch {destroy .opt}"
    pack $botbox.doit $botbox.end -side left
}
proc hideelembox {onoff box} {
    if {$onoff} {
	$box.3 config -state disabled -relief flat -fg [$box.3 cget -bg]
	$box.4 config -state disabled
	catch {destroy .opt}
    } else {
	$box.3 config -state normal -relief sunken -fg black
	$box.4 config -state normal
    }
}

proc saveboxastext {box {print ""}} {
    global settings tcl_platform 
    if [catch {
	if {$print == ""} {
	    set file [tk_getSaveFile -defaultextension .txt]
	    if {[string trim $file] == ""} return
	    set fp [open $file a]
	    puts $fp [clock format [clock seconds]]
	    puts $fp [$box get 1.0 end]
	    close $fp
	} else {
	    set file tmp.txt
	    set fp [open $file w]
	    puts $fp [clock format [clock seconds]]
	    puts $fp [$box get 1.0 end]
	    close $fp
	    if {$tcl_platform(platform) != "windows"} {
		eval exec cat $file | $settings(printer)
		file delete $file
	    } else {
		eval exec $settings(printer) $file
	    }
	}
    } errmsg ] {
	if {$errmsg == "" && $file == ""} return
	tk_dialog .msg "error writing file" \
		"Error writing file: $errmsg" \
		error 0 OK
    }
}

proc savepeaks {} {
    global settings
    if [catch {
	set file [tk_getSaveFile -defaultextension .pks]
	if {[string trim $file] == ""} return
	set fp [open $file a]
	puts $fp [clock format [clock seconds]]
	set list [getpdf1 -seq $settings(seqno) -common -mineral -chemname]
	puts -nonewline $fp "[getpdf1 -seq $settings(seqno) -entry]: "
	puts -nonewline $fp \
		"[lindex [getpdf1 -seq $settings(seqno) -formula] 0] ("
	foreach item $list {
	    if {$item != ""} {
		puts -nonewline  $fp "$item "
	    }
	}
	puts $fp ")"
	puts -nonewline $fp "d-space\tint"
	set peaks [lindex [getpdf1 -seq $settings(seqno) -peaks ] 0]
	if {$settings(peaktype) == "-q" } {
	    set peaks1 [lindex [eval getpdf1 -seq $settings(seqno) -peaks -q] 0]
	    puts $fp "\tQ"
	} elseif {$settings(peaktype) == "-wave"} {
	    set peaks1 [lindex [eval getpdf1 -seq $settings(seqno) -peaks \
		    -wave $settings(wave) ] 0]
	    puts $fp "\t2Theta, (wave = $settings(wave))"
	} else {
	    set peaks1 ""
	}
	set i 0
	while {$i < [llength $peaks]} {
	    set pos [lindex $peaks $i]
	    set tt [lindex $peaks1 $i]
	    incr i
	    set int [lindex $peaks $i]
	    puts $fp "$pos\t$int\t$tt"
	    incr i
	}
	close $fp
    } errmsg ] {
	tk_dialog .msg "error writing peaks" \
		"Error writing file: $errmsg" \
		error 0 OK
    }
}

# restart the browse window, if the window is open
proc UpdateBrowse {} {
    # see if the browse window exists
    if [catch { wm state .browse} ] {return}
    first_entry
    global settings
    set button $settings(browse_viewas)
    if {[$button cget -text] == "view as pdf2"} {
	display_entry_pdf1 
    } else {
	display_entry_pdf2
    }
}
# create the browse dialog box
proc make_browse { {first 0} } {
    global txtbx settings
    # reuse previous browse window
    set top .browse
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    set txtbx [text $top.t -relief raised -bd 2 -height 10 -width 50 \
	-wrap none \
	-yscrollcommand "$top.s set" \
	-xscrollcommand "$top.a set"]
    scrollbar $top.s -command "$top.t yview"
    scrollbar $top.a -command "$top.t xview" -orient horizontal
    frame $top.bar -relief groove -bd 2
    pack $top.bar $top.a -side bottom -fill x
    pack $top.t -side left -fill both -expand yes
    pack $top.s -side left -fill y 
    pack [button $top.bar.next -text Next] -side left
    set settings(browse_previous) $top.bar.prev
    pack [button $top.bar.prev -text Previous] -side left
    set settings(browse_viewas) $top.bar.fmt 
    pack [button $top.bar.fmt -text "view as pdf2"] -side left 

    pack [button $top.bar.saveas -text "Save as..." \
	    -command "saveboxastext $top.t "] -side left
    pack [button $top.bar.print -text "Print" \
	    -command "saveboxastext $top.t print"] -side left
    pack [button $top.bar.export -text "Export..." \
	    -command savepeaks] -side left
    pack [button $top.bar.end -text Close -command "destroy $top"] -side right
    $top.bar.fmt config -command switch_pdf_view
    $top.bar.next config -command next_entry
    $top.bar.prev config -command prev_entry 
    if $first first_entry
    display_entry_pdf1
}

proc switch_pdf_view {} {
    global settings
    set button $settings(browse_viewas)
    if {[$button cget -text] == "view as pdf2"} {
	$button config -text "view as pdf1"
	display_entry_pdf2
    } else {
	$button config -text "view as pdf2"
	display_entry_pdf1
    }
}

proc first_entry {} {
    global settings
    set settings(hitnum) 0
    set settings(seqno) [nexthit -first]
    incr settings(hitnum)
    $settings(browse_previous) config -state disabled
}

proc next_entry {} {
    global settings
    set button $settings(browse_viewas)
    set settings(seqno) [nexthit]
    incr settings(hitnum)
    if {$settings(seqno) < 0} first_entry
    if {[$button cget -text] == "view as pdf2"} {
	display_entry_pdf1 
    } else {
	display_entry_pdf2
    }
    if {$settings(hitnum) == 1} {
	$settings(browse_previous) config -state disabled
    } else {
	$settings(browse_previous) config -state normal
    }
}

proc prev_entry {} {
    global settings
    set button $settings(browse_viewas)
    set settings(seqno) [nexthit -backward]
    incr settings(hitnum) -1
    if {$settings(seqno) < 0} first_entry
    if {$settings(hitnum) == 1} {
	$settings(browse_previous) config -state disabled
    }
    if {[$button cget -text] == "view as pdf2"} {
	display_entry_pdf1 
    } else {
	display_entry_pdf2
    }
}

proc display_entry_pdf1 {} {
    global txtbx tcl_version settings
    if {$settings(peaktype) == "-wave"} {
	set list [eval getpdf1 -seq $settings(seqno) -entry -formula \
		-chemname -common -mineral -reference -peaks \
		$settings(peaktype) $settings(wave) \
		$settings(peakorder)]
    } else {
	set list [eval getpdf1 -seq $settings(seqno) -entry -formula \
		-chemname -common -mineral -reference -peaks \
		$settings(peaktype) $settings(peakorder)]
    }
    $txtbx configure -state normal
    if {$tcl_version >= 8.0} {$txtbx config -font $settings(font)}
    $txtbx delete 1.0 end
    set patnumb "[lindex $list 0]\t"
    regsub "D\t" $patnumb " (Deleted)" patnumb
    $txtbx insert end \
	    "Entry: $patnumb\t(hit $settings(hitnum)/[report -current])\n"
    set tmp [lindex $list 1] 
    if {$tmp != {}} {$txtbx insert end "Chemical formula:\t$tmp\n"}
    set tmp [lindex $list 2]
    if {$tmp != {}} {$txtbx insert end "Chemical name:\t$tmp\n"}
    set tmp [lindex $list 3]
    if {$tmp != {}} {$txtbx insert end "Common name:\t$tmp\n"}
    set tmp [lindex $list 4]
    if {$tmp != {}} {$txtbx insert end "Mineral name:\t$tmp\n"}
    set tmp [lindex $list 5]
    if {$tmp != {}} {$txtbx insert end "Reference:\t$tmp\n"}
    if {$settings(peaktype) == "-q" } {
	$txtbx insert end "Peaks, Q (A-1):\n"
    } elseif {$settings(peaktype) == "-wave"} {
	$txtbx insert end "Peaks, 2Theta, (degrees, wave = $settings(wave)):\n"
    } else {
	$txtbx insert end "Peaks, d-spaces (A):\n"
    }
    # format peaks
    set peaks [lindex $list 6]
    set ncol 4
    set nrow [expr ([llength $peaks]+ 2*$ncol - 1) / ( 2*$ncol ) ]
    for {set row 0} {$row < $nrow} {incr row} {
	$txtbx insert end "  "
	for {set col 0} {$col < $ncol} {incr col} {
	    set pos [expr 2*($row + $nrow*$col)]
	    if {$pos >= [llength $peaks]} break
	    $txtbx insert end \
		    [format "%8.3f %3d" \
		     [lindex $peaks $pos] [lindex $peaks [expr $pos+1]]]
	}
	$txtbx insert end "\n"
    }
#    $txtbx insert end "\t[lindex $list 6]\n"
    $txtbx configure -state disabled  -height 10 -width 50
}

proc display_entry_pdf2 {} {
    global txtbx tcl_version settings

    if {$settings(peaktype) == "-q" } {
	set fmt Q
    } elseif {$settings(peaktype) == "-wave"} {
	set fmt $settings(wave)
    } else {
	set fmt D
    }
    set list [getpdf2 -seq $settings(seqno) -file pdf2.fmt -format $fmt]

    set fl [open pdf2.fmt r]
    set fmt [read $fl]
    close $fl
    file delete pdf2.fmt
    $txtbx configure -state normal
    if {$tcl_version >= 8.0} {$txtbx config -font $settings(font)}
    $txtbx delete 1.0 end
    $txtbx insert end \
	    "\t\t\t\t\t\t\t\t\t\t\t\t\t\t ($settings(hitnum)/[report -current])\n"
    $txtbx insert end $fmt
    if {$settings(showaids)} {
	$txtbx insert end \n
	foreach line $list {
	    $txtbx insert end "$line\n"
	}
	set dbg [open "lastaids.rec" w]
	foreach line $list {
	    puts $dbg $line
	}
	close $dbg
    }
    $txtbx configure -state disabled -width $settings(pdf2width) -height 24  
}

proc validatewave {} {
    global settings
    set settings(temp) d
    switch -- $settings(peaktype) {
	-q {set settings(temp) q} 
	-wave {
	    set settings(temp) O
	    if {$settings(wave) == 1.5418} {set settings(temp) C}
	}
    }
}

proc testwave {top} {
    global settings
    switch $settings(temp) {
	q {set settings(peaktype) -q}
	C {
	    set settings(peaktype) -wave
	    set settings(wave) 1.5418
	}
	O {
	    if {[catch {expr $settings(wave)*1}]} {
		$top.a.warn config -text "Invalid Wavelength"; return}
	    if {$settings(wave) <= 0} {
		$top.a.warn config -text "Invalid Wavelength"; return}
	    set settings(peaktype) -wave
	}
	default { set settings(peaktype) ""}
    }
    destroy $top
}
proc settings_menu {top} {
    global settings 
    # reuse any window with this name
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    wm title $top "Display settings"
    pack [frame $top.a]
    set topa $top.a
    grid [button $topa.d -text Done -command "testwave $top"] -row 2 -column 0 -columnspan 2
    grid [frame $topa.a -bd 4 -relief groove] -row 0 -column 0 -rowspan 2 

    grid [frame $topa.b -bd 4 -relief groove] -row 0 -column 1 -sticky news
    pack [label $topa.b.1 -text "PDF-1/Index Peak Ordering"]
    pack [radiobutton  $topa.b.2 -value {} -text "Decreasing d-space" \
	    -var settings(peakorder) ] -side top -anchor w 
    pack [radiobutton  $topa.b.3 -value -maxi -text "Decreasing intensity" \
	    -var settings(peakorder) ] -side top -anchor w

    grid [frame $topa.c -bd 4 -relief groove] -row 1 -column 1 -sticky news
    pack [label $topa.c.1 -text "PDF-2 Display"]
    pack [checkbutton  $topa.c.2 -text "Include AIDS*83 records" \
	    -var settings(showaids) ] -side top -anchor w 

    validatewave
    set enable     {if {$settings(temp) == "O"} }
    append enable " \{ $topa.a.w config -state normal -relief sunken; "
    append enable  " $topa.a.w config -fg \[lindex \[$topa.a.w config -fg\] 3\]\}"
    append enable " \{ $topa.a.w config -state disabled -relief flat ;" 
    append enable   " $topa.a.w config -fg \[ $topa.a.w cget -bg \] ;"
    append enable   " $topa.a.warn config -text {} \}"
    pack [label $topa.a.l -text "Display Peaks as"] -side top
    pack [radiobutton  $topa.a.1 -value d -text d-space \
	    -command  $enable \
	    -var settings(temp) ] -side top -anchor w 
    pack [radiobutton  $topa.a.2 -value q -text "Q (A-1)" \
	    -command  $enable \
	    -var settings(temp) ] -side top -anchor w
    pack [radiobutton  $topa.a.3 -value C -text "2theta, CuKa" \
	    -command  $enable \
	    -var settings(temp) ] -side top -anchor w
    pack [radiobutton  $topa.a.4 -value O -text "2theta, other" \
	    -command  $enable \
	    -var settings(temp) ] -side top -anchor w
    pack [entry $topa.a.w -textvar settings(wave) -width 8]
    pack [label $topa.a.warn -text ""]
    eval $enable
    centerontop $top
}

proc printsettings_menu {top} {
    global settings 
    # reuse any window with this name
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    wm title $top "Print settings"
    pack [button $top.d -text Done -command "destroy $top"] -side bottom 
    pack [frame $top.a -bd 4 -relief groove] -side left 
    pack [frame $top.b -bd 4 -relief groove] -side left
    pack [label $top.b.1 -text "Printer command line"]
    pack [entry $top.b.2 -textvariable settings(printer) -width 30] -side top -anchor w 
    centerontop $top
}

proc disableindexentry {val top dummy} {
    global settings
    if {$settings(len$val) > 5 } {
	$top.f.$val.t config -state normal
    } else { 
	$top.f.$val.t config -state disabled 
    }
}

proc indexsettings_menu {top} {
    global settings 
    # reuse any window with this name
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    wm title $top "Index contents"

    pack [frame $top.a ] -side top
    pack [label $top.a.1 -text \
	    "Select the entries to include in the index and their lengths"]

    pack [frame $top.f ] -side top
    foreach val {1 2 3 4 5 6} {
	pack [frame $top.f.$val -bd 4 -relief groove] -side left -fill both
	scale $top.f.$val.s -from 0 -to 50 -orient horizontal -label length \
		-variable settings(len$val) \
		-command "disableindexentry $val $top"
	eval tk_optionMenu $top.f.$val.t settings(ent$val) $settings(options)
	$top.f.$val.t config -takefocus {}
	pack [label $top.f.$val.l -text "entry $val" -width 15] 
	pack $top.f.$val.s  $top.f.$val.t 
    }
    pack [frame $top.b ] -side bottom
    pack [button $top.b.2 -text Done \
	    -command "destroy $top;UpdateIndex"] -side right
    pack [button $top.b.1 -text Apply \
	    -command UpdateIndex] -side left
    centerontop $top
}

# writes ~/.logicrc to save options
proc savesettings {} {
    global settings
    set fp [open ~/.logicrc w]
    puts $fp "array set settings {"
    foreach var {1 2 3 4 5 6} {
	puts $fp "\t len$var \t$settings(len$var)\t ent$var [list $settings(ent$var)]"
    }
    foreach item {peakorder peaktype temp wave font printer showaids} {
	puts $fp "\t $item [list $settings($item)]"
    }
    puts $fp "}"
    close $fp
}

# index_to_browse is used to display an entry from the index by
# double clicking
proc index_to_browse {textbx coord} {
    global settings
    set linenum [lindex [split [$textbx index @$coord] .] 0]
    # jump to the selected entry
    set num [expr $linenum - 1]
    if {$num < 1} {set num 1}
    if {$num > [report -total]} {set num [report -total]}
    set settings(seqno) [nexthit [set settings(hitnum) $num]]
    if [catch {
	set button $settings(browse_viewas)
	
	if {[$button cget -text] == "view as pdf2"} {
	    display_entry_pdf1 
	} else {
	    display_entry_pdf2
	}  
    }] {make_browse}
}

# reload the index contents, if the window is open
proc UpdateIndex {} {
    # see if an index window exists
    if [catch { wm state .index} ] {return}
    make_index
}

proc make_index { } {
    global settings tcl_version indexbx

    # reuse previous index window
    set top .index
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    pack [frame $top.bar -relief groove -bd 2] -side bottom -fill x
    pack [scrollbar $top.a -command "$top.t xview" -orient horizontal] \
	    -side bottom -fill x
    set indexbx [text $top.t -relief raised -bd 2 -height 10 -width 50 \
	-wrap none \
	-yscrollcommand "$top.s set" \
	-xscrollcommand "$top.a set"]
    if {$tcl_version >= 8.0} {$indexbx config -font $settings(font)}
    bind $indexbx <Double-1> "index_to_browse $indexbx %x,%y"
    pack $indexbx -side left -fill both -expand yes
    pack [scrollbar $top.s -command "$top.t yview"] -side left -fill y 
    set settings(abort) 0
    pack [button $top.bar.stop -text stop \
	    -command "set settings(abort) 1"] -side right
    pack [label $top.bar.label -text ""] -side left
    pack [button $top.bar.saveas -text "Save as..." \
	    -command "saveboxastext $top.t "] -side left
    pack [button $top.bar.print -text "Print" \
	    -command "saveboxastext $top.t print"] -side left

    update

    set lengths {}
    set seqloc {}
    set seqlen 0
    set header {}
    set entries {}
    foreach val {1 2 3 4 5 6} {
	if { $settings(len$val) > 5 } {
	    lappend header $settings(ent$val)
	    set entnum [lsearch $settings(options) $settings(ent$val)]
	    if {$entnum == 0} {
		set seqloc [llength $lengths] 
		set seqlen $settings(len$val)
	    } else {
		lappend lengths $settings(len$val)
		lappend entries [lindex $settings(flag) $entnum]
	    }
	}
    }
    if {$seqlen > 0} {
	set  lengths [linsert $lengths $seqloc $seqlen]
    }
    # start entering info into the listbox
    $indexbx configure -state normal
    $indexbx delete 1.0 end
    # make a header
    set line {}
    for {set i 0} {$i < [llength $settings(options)]} {incr i} {
	set x [lindex $header $i]
	append x "                                                 "
	append line [string range $x 0 [expr [lindex $lengths $i]-1]] " "
    }
    $indexbx insert end $line
#    $indexbx insert end "\n"
    set seq 1
    set settings(seqno) [nexthit -first]
    set total [report -current]
    set prevdone 0
    while {$settings(seqno) > 0} {
	set done [expr 100 * $seq / $total]
	if {$done > $prevdone} {
	    set prevdone $done 
	    $top.bar.label config -text "$done % complete"
	    update
	    if {$settings(abort)} break
	}
	set line {}
	# get the pdf1 info
	if {$settings(peaktype) == "-wave"} {
	    set list [eval getpdf1 -seq $settings(seqno) $entries \
		    $settings(peaktype) $settings(wave) \
		    $settings(peakorder)]
	} else {
	    set list [eval getpdf1 -seq $settings(seqno) $entries \
		    $settings(peaktype) $settings(peakorder)]
	}
	# add the sequence number
	if {$seqlen > 0} {set list [linsert $list $seqloc $seq]}
	# create a line from the entry
	for {set i 0} {$i < [llength $list]} {incr i} {
	    set x [lindex $list $i]
	    append x "                                                 "
	    append line [string range $x 0 [expr [lindex $lengths $i]-1]] " "
	}
	set settings(seqno) [nexthit]
	incr seq
	$indexbx insert end "\n"
	$indexbx insert end $line
    }
    # remove counting label
    if {! $settings(abort)} {pack forget $top.bar.label}
    # reconfigure the stop button
    $top.bar.stop config -text Close -command "destroy $top"
    $indexbx configure -state disabled  -height 10 -width 50
    UpdateBrowse
}



#----------------------------------------------------------------
# enter information into logic (main) screen
#----------------------------------------------------------------
proc update_logic {} {
    .l2 configure -text "[report -current] of [report -total] entries"
    .l3.t configure -state normal
    .l3.t delete 1.0 end
    set num 1
    .l3.t insert end "Command history\n\n"
    foreach item [report -history] {
	.l3.t insert end " $num) $item\n"
	incr num
    }
    .l3.t configure -state disabled
    # since history is changed update browse & index
    UpdateIndex
}

proc saveundo {} {
    global settings
    storemap
    eval $settings(undo) -state normal
}

proc undo {} {
    global settings
    set before [report -current]
    restoremap
    set after [report -current]
    if {$after > 1.5*$before && $after > 1000} {catch {destroy .index .browse}}
    eval $settings(undo) -state disabled
    update_logic
}

proc peakmenudone {top logic} {
    global peakopt $logic
    switch $peakopt(temp) {
	q {set peakopt(peaktype) -q}
	C {
	    set peakopt(peaktype) -wave
	    set peakopt(wave) 1.5418
	}
	O {
	    if {[catch {expr $peakopt(wave)*1}]} {
		warnbox "Invalid Wavelength: $peakopt(wave)"; return}
	    if {$peakopt(wave) <= 0} {
		warnbox "Invalid Wavelength: $peakopt(wave)"; return}
	    set peakopt(peaktype) -wave
	}
	default { set peakopt(peaktype) ""}
    }
    set peaklist {}
    foreach var {1 2 3 4 5 6 7 8} {
	if {$peakopt(peak$var) != ""} {	
	    if {[catch {expr $peakopt(peak$var)*1}]} {
		warnbox "Invalid peak value: $peakopt(peak$var)"; return
	    }
	    if {$peakopt(peak$var) <= 0} {
		warnbox "Invalid peak value: $peakopt(peak$var)"; return
	    }
	    lappend peaklist $peakopt(peak$var)
	}
    }
    if {[llength $peaklist] <= 0} {
		warnbox "No valid peaks to search!"; return
    }
    set errorlist {}
    foreach var {1 2 3 4 5 6 7 8} {
	if {$peakopt(err$var) != ""} {	
	    if {[catch {expr $peakopt(err$var)*1}]} {
		warnbox "Invalid error value: $peakopt(err$var)"; return
	    }
	    if {$peakopt(err$var) <= 0} {
		warnbox "Invalid error value: $peakopt(err$var)"; return
	    }
	    lappend errorlist $peakopt(err$var)
	}
    }
    if {[llength $errorlist] <= 0} {
		warnbox "No valid error range!"; return
    }
    saveundo
    waitbox {Search in progress, please wait}
    update
    if {$peakopt(peaktype) == "-wave"} {
	set list [eval findpeak \
		-peak [list $peaklist] -error [list $errorlist] \
		$peakopt(0) $peakopt(1) $peakopt(3) -$[set logic] \
		$peakopt(peaktype) $peakopt(wave) ]
    } else {
	set list [eval findpeak \
		-peak [list $peaklist] -error [list $errorlist] \
		$peakopt(0) $peakopt(1) $peakopt(3) -$[set logic] \
		$peakopt(peaktype) ]
    }
    catch {destroy .wait}
    update_logic
    catch {destroy .warn}
}

proc peak_menu {top} {
    global rightbox peakopt
    # reuse any window with this name
    catch {toplevel $top}
    raise $top
    eval destroy [pack slaves $top]
    wm title $top "Select by peaks"

    set leftmid  $top.1 
    pack [frame $leftmid -relief raised -bd 2] -fill x -side top

    set enable     {if {$peakopt(temp) == "O"} }
    append enable " \{ $leftmid.w config -state normal -relief sunken; "
    append enable  " $leftmid.w config -fg \[lindex \[$leftmid.w config -fg\] 3\]\}"
    append enable " \{ $leftmid.w config -state disabled -relief flat ;" 
    append enable   " $leftmid.w config -fg \[ $leftmid.w cget -bg \] \}"
    pack [label $leftmid.l -text "Enter peaks as "] -side top -anchor w
    pack [radiobutton  $leftmid.1 -value d -text d-space \
	    -command  $enable \
	    -var peakopt(temp) ] -side left -anchor w 
    pack [radiobutton  $leftmid.2 -value q -text "Q (A-1)" \
	    -command  $enable \
	    -var peakopt(temp) ] -side left -anchor w
    pack [radiobutton  $leftmid.3 -value C -text "2theta, CuKa" \
	    -command  $enable \
	    -var peakopt(temp) ] -side left -anchor w
    pack [radiobutton  $leftmid.4 -value O -text "2theta, wavelength:" \
	    -command  $enable \
	    -var peakopt(temp) ] -side left -anchor w
    pack [entry $leftmid.w -textvar peakopt(wave) -width 8] -side left -anchor e
    eval $enable

    pack [frame $top.2 -relief raised -bd 2] -fill x -side top 
    pack [label $top.2.l -text "Enter one or more peaks:"] \
	    -side top -anchor w
    foreach var {1 2 3 4 5 6 7 8} {
	pack [entry $top.2.$var -textvar peakopt(peak$var) -width 8] \
		-side left -padx 1 -fill x -expand yes
    }
    pack [frame $top.3 -relief raised -bd 2] -fill x -side top 
    pack [label $top.3.l -text "Enter one or more errors"] \
	    -side top -anchor w
    foreach var {1 2 3 4 5 6 7 8} {
	pack [entry $top.3.$var -textvar peakopt(err$var) -width 8] \
		-side left -padx 1 -fill x -expand yes
    }

    set topbox [frame $top.a]; # frame for all boxes
    set botbox [frame $top.b]; # frame for buttons
    pack $topbox $botbox -side top
    # create frame for the options menu and then the menu
    set rightbox [frame $top.a.option -relief raised -bd 2]
    options_list $rightbox peak_result search
    pack $rightbox -side left

    set peakmid [frame $top.a.mid -relief raised -bd 2] 
    pack $peakmid -side left -fill x
    pack [label $peakmid.t -text "Peak Threshold"] -side top
    pack [radiobutton $peakmid.1 -variable peakopt(0) -value -3strongest\
	    -text "One of the 3 strongest"]\
	    -side top -anchor w
    pack [frame $peakmid.r ] -side right
    pack [frame $peakmid.l ] -side left
    foreach val {0 10 20 30 40} {
	pack [radiobutton $peakmid.l.$val -variable peakopt(0) \
		-value "-intmin $val" -text ">= $val%"]\
		-side top -anchor w
    }
    foreach val {50 60 70 80 90} {
	pack [radiobutton $peakmid.r.$val -variable peakopt(0) \
		-value "-intmin $val" -text ">= $val%"]\
		-side top -anchor w
    }
    $peakmid.l.30 invoke

    set leftbox [frame $top.a.l] 
    pack $leftbox -side left


    set lefttop [frame $top.a.ltop -relief raised -bd 2] 
    pack $lefttop -in $leftbox -side top -fill x
    pack [label $lefttop.0 -text "Multiple peaks"] -side top
    pack [radiobutton $lefttop.1 -variable peakopt(1) -value -any\
		-text "Require one of list"] -side top -anchor w
    pack [radiobutton $lefttop.2 -variable peakopt(1) -value -all\
		-text "Require all of list"] -side top -anchor w
    $lefttop.1 invoke


    set leftbot [frame $top.a.lbot -relief raised -bd 2] 
    pack $leftbot -in $leftbox -side top -fill x
    pack [label $leftbot.0 -text "Search speed"] -side top
    pack [radiobutton $leftbot.1 -variable peakopt(3) -value -fast\
	    -text "Expand errors as needed"]\
	    -side top -anchor w
    pack [radiobutton $leftbot.2 -variable peakopt(3) -value -exact\
	    -text "Use exact errors (slow)"]\
	    -side top -anchor w
    $leftbot.1 invoke

    button $botbox.doit -text Apply -command "peakmenudone $top peak_result"
    button $botbox.end -text Close \
	    -command "destroy $top; catch {destroy .warn}"
    pack $botbox.doit $botbox.end -side left
}
